package com.persagy.proxy.migration.handler;

import cn.hutool.core.collection.CollUtil;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import com.persagy.proxy.migration.constant.MigrationTable;
import com.persagy.proxy.migration.model.DataMigrationExcel;
import com.persagy.proxy.migration.model.MigrationInfo;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author lvxianyun
 * @title: DataMigrationHandler
 * @projectName adm-middleware
 * @date 2021/12/3 11:14
 */
@Component
public class DataMigrationHandler {

    private static Map<String, IDataMigration> migrationMap = new HashMap<>(16);

    public Map<String, List<DataMigrationExcel>> handleAndGetExcelList(InstanceUrlParam context, MigrationInfo migrationInfo){
        Map<String,List<DataMigrationExcel>> results = new HashMap<>(16);
        if(CollUtil.isNotEmpty(migrationMap)){
            for (MigrationTable migrationTable : MigrationTable.values()){
                if(!migrationMap.containsKey(migrationTable.getCode())){
                    continue;
                }
                results.put(migrationTable.getCode(), migrationMap.get(migrationTable.getCode()).migrateForExcel(context, migrationInfo));
            }
        }
        return results;
    }

    public void handleAndSaveLog(InstanceUrlParam context, MigrationInfo migrationInfo){
        if(CollUtil.isNotEmpty(migrationMap)){
            for (MigrationTable migrationTable : MigrationTable.values()){
                if(!migrationMap.containsKey(migrationTable.getCode())){
                    continue;
                }
                migrationMap.get(migrationTable.getCode()).migrateForLog(context, migrationInfo);
            }
        }
    }

    public void addMap(String tableName, IDataMigration iDataMigration){
        migrationMap.put(tableName, iDataMigration);
    }

    @PostConstruct
    private void initHandleService(){
        addMap(MigrationTable.DTDEFINETYPE.getCode(), DefineTypeMigration.getInstance());
        addMap(MigrationTable.DTDEFINEINFO.getCode(), DefineInfoMigration.getInstance());
        addMap(MigrationTable.DTOBJECT.getCode(), ObjectDigitalMigration.getInstance());
        addMap(MigrationTable.DTRALATION.getCode(), ObjectRelationMigration.getInstance());
    }
}
