package com.persagy.proxy.migration.handler;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.basic.utils.JsonNodeUtils;
import com.persagy.dmp.common.constant.ResponseCode;
import com.persagy.dmp.common.model.entity.BaseEntity;
import com.persagy.dmp.define.client.DigitalDefineFacade;
import com.persagy.dmp.define.entity.ObjectTypeDefine;
import com.persagy.dmp.rwd.migrate.entity.SyncData;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import com.persagy.proxy.migration.constant.MigrationConstant;
import com.persagy.proxy.migration.constant.MigrationTable;
import com.persagy.proxy.migration.constant.MigrationType;
import com.persagy.proxy.migration.model.DataMigrationExcel;
import com.persagy.proxy.migration.model.DataMigrationResponse;
import com.persagy.proxy.migration.model.MigrationInfo;
import com.persagy.proxy.migration.service.Impl.MigrationAbstractServiceImpl;
import lombok.extern.slf4j.Slf4j;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author lvxianyun
 * @projectName adm-middleware
 * @description: 迁移adm交付的数据到现场，覆盖现场数据，提供增加、删除、修改 记录放入excel; DefineType 数据迁移 adm -> target project
 * @date 2021/12/1 19:38
 */
@Slf4j
public class DefineTypeMigration extends MigrationAbstractServiceImpl<ObjectTypeDefine> implements IDataMigration, IDataMigrationProcess<ObjectTypeDefine> {

    private static DefineTypeMigration instance;

    private DefineTypeMigration() {}

    // 静态方法返回该实例
    public static DefineTypeMigration getInstance() {
        if(instance == null) {
            synchronized (DefineTypeMigration.class) {
                if (instance == null) {
                    instance = new DefineTypeMigration();
                }
            }
        }
        return instance;
    }


    /**
     * 数据迁移
     *
     * @param context
     * @param migrationInfo
     * @return
     */
    @Override
    public List<DataMigrationExcel> migrateForExcel(InstanceUrlParam context, MigrationInfo migrationInfo) {
        log.info("######################### DefineType 同步开始 #########################");
        long start = System.currentTimeMillis();
        List<DataMigrationExcel> dataMigrationExcels = new ArrayList<>();

        QueryCriteria queryCriteria = new QueryCriteria();
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        criteria.put("valid", 1);
        ObjectNode groupCode =  JsonNodeFactory.instance.objectNode();
        groupCode.put("$ne", "0");

        ObjectNode projectId =  JsonNodeFactory.instance.objectNode();
        projectId.put("$ne", "0");

        ObjectNode andOr = JsonNodeFactory.instance.objectNode();
        andOr.putPOJO("groupCode", projectId);
        andOr.putPOJO("projectId", groupCode);

        criteria.putPOJO("$andOr", andOr);

        queryCriteria.setCriteria(criteria);
        queryCriteria.setSize(-1L);

        List<ObjectTypeDefine> admDefineList = getAdmData(context, queryCriteria);
        String queryUrl = requestUrl(context, migrationInfo, MigrationType.QUERY.getCode());
        List<ObjectTypeDefine> projectDefineList = getProjectData(queryCriteria, ObjectTypeDefine.class, queryUrl);
        List<ObjectNode> objectNodeList = JsonNodeUtils.toListNode(projectDefineList, null, null);
        if(CollUtil.isEmpty(admDefineList)){
            log.error(" ######################### DefineType adm已交付需要同步的数据:"+ ResponseCode.C0320.getDesc() +" 同步结束 #########################");
            return dataMigrationExcels;
        }

        String insertUrl = requestUrl(context, migrationInfo, MigrationType.INSERT.getCode());
        if(CollUtil.isEmpty(objectNodeList)){
            log.info("######################### DefineType 批量插入数据 #########################");
            DataMigrationResponse dataMigrationResponse = insertBatch(admDefineList, ObjectTypeDefine.class, insertUrl);
            dataMigrationExcels.addAll(processDataForExcel(dataMigrationResponse, MigrationType.INSERT.getDesc()));

            return dataMigrationExcels;
        }
        List<ObjectNode> objectNodeListPro = JsonNodeUtils.toListNode(objectNodeList, null, null);
        Map<String,Object> projectDefineMap = toEntityMap(objectNodeListPro, ObjectTypeDefine.class);
        List<ObjectNode> objectNodeListAdm = JsonNodeUtils.toListNode(admDefineList, null, null);
        Map<String,Object> admDefineMap = toEntityMap(objectNodeListAdm, ObjectTypeDefine.class);

        //差集 新增
        log.info("######################### DefineType 批量处理新增数据 #########################");
        Map<String,Object> doSubtractFromInsert = doSubtractAdd(admDefineMap, projectDefineMap);
        if(!CollUtil.isEmpty(doSubtractFromInsert)){
            List<ObjectTypeDefine> insertData = toList(doSubtractFromInsert, admDefineList);
            DataMigrationResponse dataMigrationResponse = insertBatch(insertData, ObjectTypeDefine.class, insertUrl);
            dataMigrationExcels.addAll(processDataForExcel(dataMigrationResponse, MigrationType.INSERT.getDesc()));
        }

        //差集 删除
        log.info("######################### DefineType 批量处理删除数据 #########################");
        Map<String,Object> doSubtractFromDelete = doSubtractDel(admDefineMap, projectDefineMap);
        if(!CollUtil.isEmpty(doSubtractFromDelete)){
            List<String> deleteIds = doSubtractFromDelete.entrySet().stream().map(en -> en.getKey()).collect(Collectors.toList());
            List<ObjectTypeDefine> deleteData = toList(doSubtractFromDelete, projectDefineList);

            String delUrl = requestUrl(context, migrationInfo, MigrationType.DELETE.getCode());
            DataMigrationResponse dataMigrationResponse = deleteBatch(deleteIds, ObjectTypeDefine.class, delUrl);
            dataMigrationExcels.addAll(processDataForExcel(dataMigrationResponse, MigrationType.DELETE.getDesc()));
        }

        //交集更新
        log.info("######################### DefineType 批量处理更新数据 #########################");
        Map<String,Object> intersectionUpdateFrom = doIntersectionGetFrom(admDefineMap, projectDefineMap);
        Map<String,Object> intersectionUpdateTo = doIntersectionGetTo(admDefineMap, projectDefineMap);

        if(!CollUtil.isEmpty(intersectionUpdateFrom) && !CollUtil.isEmpty(intersectionUpdateTo)){
            Map<String, Object> compareData = compareData(intersectionUpdateFrom, intersectionUpdateTo);
            if(CollUtil.isNotEmpty(compareData) && compareData.size() > 0){
                List<ObjectTypeDefine> updateData = toList(compareData, admDefineList);
                String updateUrl = requestUrl(context, migrationInfo, MigrationType.UPDATE.getCode());
                DataMigrationResponse dataMigrationResponse = updateBatch(updateData, ObjectTypeDefine.class, updateUrl);
                dataMigrationExcels.addAll(processDataForExcel(dataMigrationResponse, MigrationType.UPDATE.getDesc()));
            }
        }
        long end = System.currentTimeMillis();
        log.info("######################### DefineType 数据迁移已结束 时间："+(end-start)+" #########################");
        return dataMigrationExcels;
    }

    @Override
    public AdmResponse migrateForLog(InstanceUrlParam context, MigrationInfo migrationInfo) {
        log.info("######################### DefineType 同步开始 #########################");
        long start = System.currentTimeMillis();
        List<SyncData> syncDataList = new ArrayList<>();

        QueryCriteria queryCriteria = new QueryCriteria();
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        criteria.put("valid", 1);
        ObjectNode groupCode =  JsonNodeFactory.instance.objectNode();
        groupCode.put("$ne", "0");

        ObjectNode projectId =  JsonNodeFactory.instance.objectNode();
        projectId.put("$ne", "0");

        ObjectNode andOr = JsonNodeFactory.instance.objectNode();
        andOr.putPOJO("groupCode", projectId);
        andOr.putPOJO("projectId", groupCode);

        criteria.putPOJO("$andOr", andOr);

        queryCriteria.setCriteria(criteria);
        queryCriteria.setSize(-1L);

        List<ObjectTypeDefine> admDefineList = getAdmData(context, queryCriteria);
        String queryUrl = requestUrl(context, migrationInfo, MigrationType.QUERY.getCode());
        List<ObjectTypeDefine> projectDefineList = getProjectData(queryCriteria, ObjectTypeDefine.class, queryUrl);
        List<ObjectNode> objectNodeList = JsonNodeUtils.toListNode(projectDefineList, null, null);
        if(CollUtil.isEmpty(admDefineList)){
            log.error(" ######################### DefineType adm已交付需要同步的数据:"+ ResponseCode.C0320.getDesc() +" 同步结束 #########################");
        }

        String insertUrl = requestUrl(context, migrationInfo, MigrationType.INSERT.getCode());
        if(CollUtil.isEmpty(objectNodeList)){
            log.info("######################### DefineType 批量插入数据 #########################");
            DataMigrationResponse dataMigrationResponse = insertBatch(admDefineList, ObjectTypeDefine.class, insertUrl);
            syncDataList.addAll(processDataForLog(dataMigrationResponse, MigrationType.INSERT.getDesc()));
        }
        List<ObjectNode> objectNodeListPro = JsonNodeUtils.toListNode(objectNodeList, null, null);
        Map<String,Object> projectDefineMap = toEntityMap(objectNodeListPro, ObjectTypeDefine.class);
        List<ObjectNode> objectNodeListAdm = JsonNodeUtils.toListNode(admDefineList, null, null);
        Map<String,Object> admDefineMap = toEntityMap(objectNodeListAdm, ObjectTypeDefine.class);

        //差集 新增
        log.info("######################### DefineType 批量处理新增数据 #########################");
        Map<String,Object> doSubtractFromInsert = doSubtractAdd(admDefineMap, projectDefineMap);
        if(!CollUtil.isEmpty(doSubtractFromInsert)){
            List<ObjectTypeDefine> insertData = toList(doSubtractFromInsert, admDefineList);
            DataMigrationResponse dataMigrationResponse = insertBatch(insertData, ObjectTypeDefine.class, insertUrl);
            syncDataList.addAll(processDataForLog(dataMigrationResponse, MigrationType.INSERT.getDesc()));
        }

        //差集 删除
        log.info("######################### DefineType 批量处理删除数据 #########################");
        Map<String,Object> doSubtractFromDelete = doSubtractDel(admDefineMap, projectDefineMap);
        if(!CollUtil.isEmpty(doSubtractFromDelete)){
            List<String> deleteIds = doSubtractFromDelete.entrySet().stream().map(en -> en.getKey()).collect(Collectors.toList());

            String delUrl = requestUrl(context, migrationInfo, MigrationType.DELETE.getCode());
            DataMigrationResponse dataMigrationResponse = deleteBatch(deleteIds, ObjectTypeDefine.class, delUrl);
            List<String> successIds = (List<String>) dataMigrationResponse.getData();
            List<ObjectTypeDefine> delObjs = toListByIds(successIds, admDefineList);
            dataMigrationResponse.setData(delObjs);
            syncDataList.addAll(processDataForLog(dataMigrationResponse, MigrationType.DELETE.getDesc()));
        }

        //交集更新
        log.info("######################### DefineType 批量处理更新数据 #########################");
        Map<String,Object> intersectionUpdateFrom = doIntersectionGetFrom(admDefineMap, projectDefineMap);
        Map<String,Object> intersectionUpdateTo = doIntersectionGetTo(admDefineMap, projectDefineMap);

        if(!CollUtil.isEmpty(intersectionUpdateFrom) && !CollUtil.isEmpty(intersectionUpdateTo)){
            Map<String, Object> compareData = compareData(intersectionUpdateFrom, intersectionUpdateTo);
            if(CollUtil.isNotEmpty(compareData) && compareData.size() > 0){
                List<ObjectTypeDefine> updateData = toList(compareData, admDefineList);
                String updateUrl = requestUrl(context, migrationInfo, MigrationType.UPDATE.getCode());
                DataMigrationResponse dataMigrationResponse = updateBatch(updateData, ObjectTypeDefine.class, updateUrl);
                syncDataList.addAll(processDataForLog(dataMigrationResponse, MigrationType.UPDATE.getDesc()));
            }
        }
        long end = System.currentTimeMillis();
        log.info("######################### DefineType 数据迁移已结束 时间："+(end-start)+" 共计："+syncDataList.size()+"条 #########################");
        super.addSynLog(context, syncDataList);
        return AdmResponse.success();
    }

    /**
     * 获取adm交付的数据，默认获取有效数据
     *
     * @param context
     * @return
     */
    @Override
    public List<ObjectTypeDefine> getAdmData(InstanceUrlParam context, QueryCriteria queryCriteria) {
        log.info("######################### DefineType 获取adm交付数据 #########################");
        List<ObjectTypeDefine> types = DigitalDefineFacade.queryObjectType(context.getGroupCode(),context.getProjectId(),context.getUserId(),context.getUserId(),queryCriteria);
        return types;
    }

    /**
     * 根据map转为需要处理的对象集合
     * 使用 MigrationConstant.spliter 作为拼接符
     * @param fromMap
     * @param fromList
     * @return
     */

    public List<ObjectTypeDefine> toList(Map<String, Object> fromMap, List<ObjectTypeDefine> fromList) {
        if(CollUtil.isEmpty(fromMap) || CollUtil.isEmpty(fromList)){
            return Collections.emptyList();
        }
        return toListByIds(fromMap.keySet(), fromList);
    }

    public List<ObjectTypeDefine> toListByIds(Collection<String> ids, List<ObjectTypeDefine> fromList) {
        if(CollUtil.isEmpty(ids) || CollUtil.isEmpty(fromList)){
            return Collections.emptyList();
        }
        List<ObjectTypeDefine> results = new ArrayList<>(ids.size());
        Map<String, ObjectTypeDefine> objectTypeDefineMap = CollUtil.fieldValueMap(fromList, BaseEntity.PROP_ID);
        for(String key : ids){
            results.add(objectTypeDefineMap.get(StrUtil.trim(key)));
        }
        return results;
    }


    /**
     * 拼接访问链接
     *
     * @param context
     * @param migrationInfo
     * @param type
     * @return
     */
    @Override
    public String requestUrl(InstanceUrlParam context, MigrationInfo migrationInfo, String type) {
        if(StrUtil.isBlank(migrationInfo.getTargetUrl())){
            return null;
        }
        StringBuilder url = new StringBuilder(migrationInfo.getTargetUrl());
        if(type.equals(MigrationType.QUERY.getCode())) {
            url.append(MigrationConstant.DEF_CLASS);
        }else if(type.equals(MigrationType.INSERT.getCode())){
            url.append(MigrationConstant.DEF_CLASS_INSERT);
        }else if(type.equals(MigrationType.UPDATE.getCode())){
            url.append(MigrationConstant.DEF_CLASS_UPDATE);
        }else{
            url.append(MigrationConstant.DEF_CLASS_DELETE);
        }
        return commonUrl(context, url.toString(), migrationInfo);
    }

    /**
     * 将目标集合处理成excel格式的集合  使用与insert/delete
     *
     * @param dataMigrationResponse
     * @return
     */
    @Override
    public List<DataMigrationExcel> processDataForExcel(DataMigrationResponse dataMigrationResponse, String type) {
        if(CollUtil.isEmpty(dataMigrationResponse.getData()) && CollUtil.isEmpty(dataMigrationResponse.getErrorMsg())){
            return Collections.emptyList();
        }
        List<DataMigrationExcel> dataMigrationExcels = new ArrayList<>();
        if(CollUtil.isNotEmpty(dataMigrationResponse.getData())){
            for(Object obj : dataMigrationResponse.getData()){
                if(!type.equals(MigrationType.DELETE.getCode())){
                    ObjectTypeDefine objectTypeDefine = (ObjectTypeDefine) obj;
                    DataMigrationExcel excel = new DataMigrationExcel();
                    excel.setId(objectTypeDefine.getId());
                    excel.setCode(objectTypeDefine.getCode());
                    excel.setName(objectTypeDefine.getName());
                    excel.setType(type);
                    dataMigrationExcels.add(excel);
                }else{
                    DataMigrationExcel excel = new DataMigrationExcel();
                    excel.setId(obj.toString());
                    excel.setType(type);
                    dataMigrationExcels.add(excel);
                }
            }
        }
        if(CollUtil.isNotEmpty(dataMigrationResponse.getErrorMsg())){
            dataMigrationExcels.addAll(addErrorInfo(dataMigrationResponse.getErrorMsg()));
        }
        return dataMigrationExcels;
    }

    /**
     * 将目标集合处理成SyncData(日志)格式的集合  入库
     *
     * @param dataMigrationResponse
     * @param type                  insert/delete
     * @return
     */
    @Override
    public List<SyncData> processDataForLog(DataMigrationResponse dataMigrationResponse, String type) {
        if(CollUtil.isEmpty(dataMigrationResponse.getData()) && CollUtil.isEmpty(dataMigrationResponse.getErrorMsg())){
            return Collections.emptyList();
        }
        List<SyncData> syncDataList = new ArrayList<>();
        if(CollUtil.isNotEmpty(dataMigrationResponse.getData())){
            for(Object obj : dataMigrationResponse.getData()){
                ObjectTypeDefine objectTypeDefine = (ObjectTypeDefine) obj;
                SyncData syncData = new SyncData();
                syncData.setTableName(MigrationTable.DTDEFINETYPE.getCode());
                syncData.setTargetId(objectTypeDefine.getId());
                syncData.setName(objectTypeDefine.getName());
                syncData.setCode(objectTypeDefine.getCode());
                syncData.setObjType(objectTypeDefine.getObjType());
                syncData.setOps(type);//操作
                syncDataList.add(syncData);
            }
        }
        if(CollUtil.isNotEmpty(dataMigrationResponse.getErrorMsg())){
            syncDataList.addAll(addErrorInfoForLog(dataMigrationResponse.getErrorMsg()));
        }
        return syncDataList;
    }

}
